#Copyright (C) 1991-2006 Altera Corporation
#Your use of Altera Corporation's design tools, logic functions 
#and other software and tools, and its AMPP partner logic 
#functions, and any output files any of the foregoing 
#including device programming or simulation files), and any 
#associated documentation or information are expressly subject 
#to the terms and conditions of the Altera Program License 
#Subscription Agreement

# The complete list and details for Tcl APIs are provided in Scripting Reference of Quartus II Scripting Reference Manual.
# Hardware required to communicate with the FPGA through JTAG port is USB Blaster (or any other download cable from Altera Corp.)

################################## Description of script #################################################################
## This script is used to show how contents of internal state machines are sampled and loaded with new values
## The internal state machines are 2 internal counters running at ~1Hz speed
## User will use the VJI instances to sample the contents of the counter and then load user specified values into both counters 

##  We have instantiated a 2 separate VJI megafunctions in the design and connected to counter1 and counter2 
##################################################################################################################################


#### Script begins ######################################################

set loop 3

# List all available programming hardwares, and select the USBBlaster.
# (Note: this example assumes only one USBBlaster connected.)
puts "Programming Hardwares:"
foreach hardware_name [get_hardware_names] {
	puts $hardware_name
	if { [string match "ByteBlasterMV*" $hardware_name] } {
		set blaster_name $hardware_name
	}
}
puts "\nSelect JTAG chain connected to $blaster_name.\n";

# List all devices on the chain, and select the first device on the chain.
puts "\nDevices on the JTAG chain:"
foreach device_name [get_device_names -hardware_name $blaster_name] {
	puts $device_name
	if { [string match "@1*" $device_name] } {
		set test_device $device_name
	}
}
puts "\nSelect device: $test_device.\n";

# Open device
open_device -hardware_name $blaster_name -device_name $test_device

# Retrieve device id code.
# IDCODE instruction value is 6; The ID code is 32 bits long.

# IR and DR shift should be locked together to ensure that other applications
# will not change the instruction register before the id code value is shifted
# out while the instruction register is still holding the IDCODE instruction.
device_lock -timeout 10000
device_ir_shift -ir_value 6 -no_captured_ir_value
puts "IDCODE: 0x[device_dr_shift -length 32 -value_in_hex]"
device_unlock

# SAMPLE instruction samples a 8-bit bus; the captured value shows the number of sample performed.
# FEED instruction supplies a 8-bit value to the logic connected to this instance.
# Both data registers corresponding to the IR are 8 bit wide.

# Send SAMPLE instruction to IR, read captured IR for the sampling number.
# Capture the DR register for the current sampled value.
set run_script 0
while {$run_script != $loop} {
set run_script [expr $run_script +1]
set counter1 0
set counter2 1

device_lock -timeout 10000
while {$counter1!=$counter2} {
device_virtual_ir_shift -instance_index 0 -ir_value 1
set counter1 [device_virtual_dr_shift -instance_index 0 -length 4 -value_in_hex]
device_virtual_ir_shift -instance_index 1 -ir_value 1
set counter2 [device_virtual_dr_shift -instance_index 1 -length 4 -value_in_hex]
puts "Value of {counter2,counter1} is <$counter2,$counter1>"
set delay 0
while {$delay != 120000} {
	set delay [expr $delay+1]
}
puts ""
}

## instead of stopping at the equal value, force a value of supplied by the user in both  counters and then end.
device_unlock
# Send FEED instruction to IR, read a two-digit hex string from the console,
# then send the new value to the DR register.
puts "\nType in a digit in hexadecimal to update the contents of the counters:"
gets stdin update_value
set update_value2 [expr $update_value+1]
device_lock -timeout 10000
device_virtual_ir_shift -instance_index 0 -ir_value 2 -no_captured_ir_value
device_virtual_dr_shift -instance_index 0  -length 4 -dr_value $update_value -value_in_hex -no_captured_dr_value
device_virtual_ir_shift -instance_index 1 -ir_value 2 -no_captured_ir_value
device_virtual_dr_shift -instance_index 1  -length 4 -dr_value $update_value2 -value_in_hex -no_captured_dr_value
device_unlock
}
# Close device
close_device

