#Copyright (C) 1991-2006 Altera Corporation
#Your use of Altera Corporation's design tools, logic functions 
#and other software and tools, and its AMPP partner logic 
#functions, and any output files any of the foregoing 
#including device programming or simulation files), and any 
#associated documentation or information are expressly subject 
#to the terms and conditions of the Altera Program License 
#Subscription Agreement

# This script gives an example of how debugging utilities could be built
# Tcl APIs commuicate with sld_virtual_jtag instances in the System On a Porgrammable Chip (SOPC) design
# The complete list and details for these APIs are provided in Quartus II Scripting Reference Manual.

# Hardware required to communicate with the FPGA through JTAG port is  Blaster (or any other download cable from Altera Corp.)


#### Script begins ######################################################

#### these variable will control the name of the file you want to write
set file1 dump

### this variable will control the delay you want to add while reading the memeory contents
## if you add a delay eg of 120000 you can visually see the contents of the memory on the seven segment display while 
## the contents are being captured from the JTAG port.

#set delay_value 0
set delay_value 12000

### these vaiables set the lower and upper limit for addresses of the memory to be read
### therefore from 0 to 255 
set lower_limit 0
set upper_limit 255

# List all available programming hardwares, and select the Blaster.
# (Note: this example assumes only one Blaster connected.)
puts "Programming Hardwares:"
foreach hardware_name [get_hardware_names] {
	puts $hardware_name
	if { [string match "ByteBlasterMV*" $hardware_name] } {
		set blaster_name $hardware_name
	}
}
puts "\nSelect JTAG chain connected to $blaster_name.\n";

# List all devices on the chain, and select the first device on the chain.
puts "\nDevices on the JTAG chain:"
foreach device_name [get_device_names -hardware_name $blaster_name] {
	puts $device_name
	if { [string match "@1*" $device_name] } {
		set test_device $device_name
	}
}
puts "\nSelect device: $test_device.\n";

# Open device 
open_device -hardware_name $blaster_name -device_name $test_device

# Retrieve device id code.
# IDCODE instruction value is 6; The ID code is 32 bits long.
# IR and DR shift should be locked together to ensure that other applications 
# will not change the instruction register before the id code value is shifted
# out while the instruction register is still holding the IDCODE instruction.
device_lock -timeout 10000
device_ir_shift -ir_value 6 -no_captured_ir_value
puts "IDCODE: 0x[device_dr_shift -length 32 -value_in_hex]"
device_unlock

puts " ******************* LOOP ************************ "
puts " "
puts " ---- contents written to ${file1}.hex"
puts " ************************************************** "
set file_id [open ${file1}.hex w+]
set counter2 ""
set adr_cntr $lower_limit
device_lock -timeout 10000
device_virtual_ir_shift -instance_index 2 -ir_value 1
device_virtual_dr_shift -instance_index 2  -length 8 -dr_value [string range [format "%#04X" $adr_cntr] 2 3] -value_in_hex -no_captured_dr_value
while {$adr_cntr != [expr $upper_limit + 1]} {
device_virtual_ir_shift -instance_index 2 -ir_value 2
set counter2 [device_virtual_dr_shift -instance_index 2 -length 8 -value_in_hex]
puts "Address counter is $adr_cntr"
puts "Value of counter2 is $counter2"
puts ""
puts $file_id "Value: $counter2" 
set adr_cntr [expr $adr_cntr + 1]
device_virtual_ir_shift -instance_index 2 -ir_value 1
device_virtual_dr_shift -instance_index 2  -length 8 -dr_value [string range [format "%#04X" $adr_cntr] 2 3] -value_in_hex -no_captured_dr_value
set delay $delay_value
while {$delay != 0} {
	 set delay [expr $delay - 1]
}
#puts ""
}
device_unlock
close $file_id

