library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

library altera_mf;
use altera_mf.all;


entity dsgn_exmple_1 is
   generic (
--      TICKS_PER_SEC : natural := 100;    -- simulation
      TICKS_PER_SEC : natural := 50333000;   -- Synthese
      CLOCK_1_SEC_COUNTER_WIDTH : natural := 32
   );

   port (
      clock         : in  std_logic;
      counter1_out  : out std_logic_vector(3 downto 0);
      counter2_out  : out std_logic_vector(3 downto 0);

      reconfig_req  : inout  std_logic 

   );
end dsgn_exmple_1;



architecture Verhalten of dsgn_exmple_1 is

   component sld_virtual_jtag
      generic(
         sld_auto_instance_index  : STRING;
         sld_instance_index       : NATURAL;
         sld_ir_width             : NATURAL;
         sld_sim_action           : STRING;
         sld_sim_n_scan           : NATURAL;
         sld_sim_total_length     : NATURAL;
         lpm_type                 : STRING
      );

      port(
         ir_out                   : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
         tdo                      : IN STD_LOGIC ;
         ir_in                    : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
         jtag_state_cdr           : OUT STD_LOGIC ;
         jtag_state_cir           : OUT STD_LOGIC ;
         jtag_state_e1dr          : OUT STD_LOGIC ;
         jtag_state_e1ir          : OUT STD_LOGIC ;
         jtag_state_e2dr          : OUT STD_LOGIC ;
         jtag_state_e2ir          : OUT STD_LOGIC ;
         jtag_state_pdr           : OUT STD_LOGIC ;
         jtag_state_pir           : OUT STD_LOGIC ;
         jtag_state_rti           : OUT STD_LOGIC ;
         jtag_state_sdr           : OUT STD_LOGIC ;
         jtag_state_sdrs          : OUT STD_LOGIC ;
         jtag_state_sir           : OUT STD_LOGIC ;
         jtag_state_sirs          : OUT STD_LOGIC ;
         jtag_state_tlr           : OUT STD_LOGIC ;
         jtag_state_udr           : OUT STD_LOGIC ;
         jtag_state_uir           : OUT STD_LOGIC ;
         tck                      : OUT STD_LOGIC ;
         tdi                      : OUT STD_LOGIC ;
         tms                      : OUT STD_LOGIC ;
         virtual_state_cdr        : OUT STD_LOGIC ;
         virtual_state_cir        : OUT STD_LOGIC ;
         virtual_state_e1dr       : OUT STD_LOGIC ;
         virtual_state_e2dr       : OUT STD_LOGIC ;
         virtual_state_pdr        : OUT STD_LOGIC ;
         virtual_state_sdr        : OUT STD_LOGIC ;
         virtual_state_udr        : OUT STD_LOGIC ;
         virtual_state_uir        : OUT STD_LOGIC
      );
   end component;

   --for jtag_inst0 : sld_virtual_jtag use worke.sld_virtual_jtag(??????????);
   --for jtag_inst1 : sld_virtual_jtag use worke.sld_virtual_jtag(??????????);

   signal clock_1_sec_ena            : std_logic;
   signal clock_1_sec_counter        : std_logic_vector(CLOCK_1_SEC_COUNTER_WIDTH-1 downto 0);

   signal counter1, counter2         : std_logic_vector(3 downto 0);
   signal feed_reg, feed_reg_2       : std_logic_vector(3 downto 0);
   signal offload_reg, offload_reg_2 : std_logic_vector(3 downto 0);

   signal ir_in                      : std_logic_vector(1 downto 0);
   signal tdi, tdo, tck              : std_logic;
   signal cdr, cir, e1dr, e2dr       : std_logic;
   signal pdr, sdr, udr, uir         : std_logic;
   
   signal sample, feed               : std_logic;
   signal bypass_reg                 : std_logic;
   signal sample_instr_counter       : std_logic_vector(1 downto 0);

   signal ir_in_2                      : std_logic_vector(1 downto 0);
   signal tdi_2, tdo_2, tck_2          : std_logic;
   signal cdr_2, cir_2, e1dr_2, e2dr_2 : std_logic;
   signal pdr_2, sdr_2, udr_2, uir_2   : std_logic;
   
   signal sample_2, feed_2             : std_logic;
   signal bypass_reg_2                 : std_logic;
   signal sample_instr_counter_2       : std_logic_vector(1 downto 0);


begin

   reconfig_req <= '1';

   con_time: process(clock)
   begin
      if clock'event and clock = '1' then
         if clock_1_sec_counter = TICKS_PER_SEC then
            clock_1_sec_ena <= '1';
            clock_1_sec_counter <= (others => '0');
         else
            clock_1_sec_ena <= '0';
            clock_1_sec_counter <= clock_1_sec_counter + 1 ;
         end if;
      end if;
   end process;

   conter: process(clock)
   begin
      if clock'event and clock = '1' then
         if feed = '1' and e1dr = '1' then
            counter1 <= feed_reg;
         elsif clock_1_sec_ena = '1' and ( counter1 /= counter2 or counter1 = 0 ) then
            counter1 <= counter1 + 1 ;
         end if;

         if feed_2 = '1' and e1dr_2 = '1' then
            counter2 <= feed_reg_2;
         elsif clock_1_sec_ena = '1' and ( counter1 /= counter2 or counter2 = 0 ) then
            counter2 <= counter1 + counter2 + 2;
         end if;

      end if;
   end process;

   counter1_out <= counter1;
   counter2_out <= counter2;


   jtag_0: process(tck)
   begin
      if tck'event and tck = '1' then

          -- Sample Instruction Handler */
         if sample = '1' and uir = '1' then
            --shadow_reg <= counter1;
            sample_instr_counter <= sample_instr_counter + 1;
         end if;

         if sample = '1' and cdr = '1' then
            offload_reg <= counter1;
         elsif sample = '1' and sdr = '1' then
            offload_reg <= tdi & offload_reg(3 downto 1);
         end if;

         -- Feed Instruction Handler */
         if feed = '1' and sdr = '1' then
            feed_reg <= tdi & feed_reg(3 downto 1);
         end if;

         -- Bypass register */
         bypass_reg <= tdi;
      end if;
   end process;

   process(sample, feed, offload_reg(0), feed_reg(0), bypass_reg)
      -- Node TDO Output */
      if sample = '1' then
         tdo <= offload_reg(0);
      elsif feed = '1' then
         tdo <= feed_reg(0);            -- Used to maintain the continuity of the scan chain.
      else
         tdo <= bypass_reg;
      end if;
   end process;


   jtag_1: process(tck)
   -- variable bypass_reg : std_logic;
   begin
      if tck_2'event and tck_2 = '1' then

         -- Sample Instruction Handler */
         if sample_2 = '1' and uir_2 = '1' then
            --shadow_reg <= counter1;
            sample_instr_counter_2 <= sample_instr_counter_2 + 1;
         end if;

         if sample_2 = '1' and cdr_2 = '1' then
            offload_reg_2 <= counter2;
         elsif sample_2 = '1' and sdr_2 = '1' then
            offload_reg_2 <= tdi_2 & offload_reg_2(3 downto 1);
         end if;

         -- Feed Instruction Handler */
         if feed_2 = '1' and sdr_2 = '1' then
            feed_reg_2 <= tdi_2 & feed_reg_2(3 downto 1);
         end if;

         -- Bypass register */
         bypass_reg_2 <= tdi_2;

      end if;
   end process;

   process(sample_2, feed_2, offload_reg_2(0), feed_reg_2(0), bypass_reg_2)
      -- Node TDO Output */
      if sample_2 = '1' then
         tdo_2 <= offload_reg_2(0);
      elsif feed_2 = '1' then
         tdo_2 <= feed_reg_2(0);        -- Used to maintain the continuity of the scan chain.
      else
         tdo_2 <= bypass_reg_2;
      end if;
   end process;


   -- instantiation of the vji mega functionc */
   jtag_inst0: sld_virtual_jtag
   generic map (
      sld_auto_instance_index => "NOT",
      sld_instance_index => 0,
      sld_ir_width => 2,
      sld_sim_action => "",
      sld_sim_n_scan => 0,
      sld_sim_total_length => 0,
      lpm_type => "sld_virtual_jtag"
   )

   port map(
      ir_out => "00",           -- input to megafunction
      ir_in => ir_in,           -- output from mega function

      tdo => tdo,               --input to mega function
      tck => tck,               -- output from mega function
      tdi => tdi,               -- output from mgafunction

      virtual_state_cdr  => cdr,  -- output from mega function
      virtual_state_e1dr => e1dr, -- "
      virtual_state_e2dr => e2dr, -- "
      virtual_state_pdr  => pdr,  -- "
      virtual_state_sdr  => sdr,  -- "
      virtual_state_udr  => udr,  -- "

      virtual_state_uir  => uir,  -- "
      virtual_state_cir  => cir   -- "
   );

   sample <= ir_in(0);
   feed   <= ir_in(1);


   -- instantiation of the vji mega functionc */
   jtag_inst1: sld_virtual_jtag
   generic map (
      sld_auto_instance_index => "NOT",
      sld_instance_index => 1,
      sld_ir_width => 2,
      sld_sim_action => "",
      sld_sim_n_scan => 0,
      sld_sim_total_length => 0,
      lpm_type => "sld_virtual_jtag"
   )

   port map(
      ir_out => sample_instr_counter_2, -- input to megafunction
      ir_in => ir_in_2,                 -- output from mega function

      tdo => tdo_2,                  --input to mega function
      tck => tck_2,                  -- output from mega function
      tdi => tdi_2,                  -- output from mgafunction

      virtual_state_cdr  => cdr_2,	 -- output from mega function
      virtual_state_e1dr => e1dr_2,  --	"
      virtual_state_e2dr => e2dr_2,  --	"
      virtual_state_pdr  => pdr_2,   --	"
      virtual_state_sdr  => sdr_2,   --	"
      virtual_state_udr  => udr_2,   --	"

      virtual_state_uir  => uir_2,   --	"
      virtual_state_cir  => cir_2    --	"
   );

   sample_2 <= ir_in_2(0);
   feed_2   <= ir_in_2(1);

end Verhalten;