library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

library altera_mf;
use altera_mf.all;


entity dsgn_exmple_2 is
   port (
      clock         : in  std_logic;
      counter1_out  : out std_logic_vector(7 downto 0);
      counter2_out  : out std_logic_vector(7 downto 0);

      reconfig_req  : inout  std_logic  -- = 1

   );
end dsgn_exmple_2;



architecture Verhalten of dsgn_exmple_2 is

   component sld_virtual_jtag
      generic(
         sld_auto_instance_index : STRING;
         sld_instance_index      : NATURAL;
         sld_ir_width            : NATURAL;
         sld_sim_action	         : STRING;
         sld_sim_n_scan	         : NATURAL;
         sld_sim_total_length    : NATURAL;
         lpm_type                : STRING
      );

      port(
         ir_out                  : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
         tdo                     : IN STD_LOGIC ;
         ir_in                   : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
         jtag_state_cdr          : OUT STD_LOGIC ;
         jtag_state_cir          : OUT STD_LOGIC ;
         jtag_state_e1dr         : OUT STD_LOGIC ;
         jtag_state_e1ir         : OUT STD_LOGIC ;
         jtag_state_e2dr         : OUT STD_LOGIC ;
         jtag_state_e2ir         : OUT STD_LOGIC ;
         jtag_state_pdr          : OUT STD_LOGIC ;
         jtag_state_pir          : OUT STD_LOGIC ;
         jtag_state_rti          : OUT STD_LOGIC ;
         jtag_state_sdr          : OUT STD_LOGIC ;
         jtag_state_sdrs         : OUT STD_LOGIC ;
         jtag_state_sir          : OUT STD_LOGIC ;
         jtag_state_sirs         : OUT STD_LOGIC ;
         jtag_state_tlr          : OUT STD_LOGIC ;
         jtag_state_udr          : OUT STD_LOGIC ;
         jtag_state_uir          : OUT STD_LOGIC ;
         tck                     : OUT STD_LOGIC ;
         tdi                     : OUT STD_LOGIC ;
         tms                     : OUT STD_LOGIC ;
         virtual_state_cdr       : OUT STD_LOGIC ;
         virtual_state_cir       : OUT STD_LOGIC ;
         virtual_state_e1dr      : OUT STD_LOGIC ;
         virtual_state_e2dr      : OUT STD_LOGIC ;
         virtual_state_pdr       : OUT STD_LOGIC ;
         virtual_state_sdr       : OUT STD_LOGIC ;
         virtual_state_udr       : OUT STD_LOGIC ;
         virtual_state_uir       : OUT STD_LOGIC
      );
   end component;


   component altsyncram
      generic (
         address_aclr_a          : STRING;
         address_aclr_b          : STRING;
         address_reg_b           : STRING;
         indata_aclr_a           : STRING;
         intended_device_family  : STRING;
         lpm_type                : STRING;
         numwords_a              : NATURAL;
         numwords_b              : NATURAL;
         operation_mode          : STRING;
         outdata_aclr_b          : STRING;
         outdata_reg_b           : STRING;
         power_up_uninitialized  : STRING;
         rdcontrol_aclr_b        : STRING;
         rdcontrol_reg_b         : STRING;
         widthad_a               : NATURAL;
         widthad_b               : NATURAL;
         width_a                 : NATURAL;
         width_b                 : NATURAL;
         width_byteena_a         : NATURAL;
         wrcontrol_aclr_a        : STRING
      );
	
      port (
         wren_a                  : IN STD_LOGIC ;
         clock0                  : IN STD_LOGIC ;
         clock1                  : IN STD_LOGIC ;
         address_a               : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
         address_b               : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
         rden_b                  : IN STD_LOGIC ;
         q_b                     : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
         data_a                  : IN STD_LOGIC_VECTOR (7 DOWNTO 0)
      );
   end component;

   --for jtag_inst0 : sld_virtual_jtag use worke.sld_virtual_jtag(??????????);
  
   signal counter1, counter2     : std_logic_vector(7 downto 0);
   signal q                      : std_logic_vector(7 downto 0);
 
   signal ir_in                  : std_logic_vector(1 downto 0);
   signal tdi, tdo, tck          : std_logic;
   signal cdr, cir, e1dr, e2dr   : std_logic;
   signal pdr, sdr, udr, uir     : std_logic;
   
   signal bypass_reg             : std_logic;
   signal sample_instr_counter   : std_logic_vector(1 downto 0);

   signal uppload_ir, adr_dr     : std_logic;
   signal read_dr, feed_dr       : std_logic;
   signal addr_reg, tmp_reg      : std_logic_vector(7 downto 0);
   signal seed                   : std_logic_vector(3 downto 0);

begin

   reconfig_req <= '1';
      
   decoder: process(ir_in)
   begin
      uppload_ir <= '0' ;
      adr_dr     <= '0' ;
      read_dr    <= '0' ;
      feed_dr    <= '0' ;     

      case ir_in is
         when b"00" => uppload_ir <= '1' ;
         when b"01" => adr_dr     <= '1' ;
         when b"10" => read_dr    <= '1' ;
         when b"11" => feed_dr    <= '1' ;
      end case;
   end process;

	
   conter: process(clock)
   begin
      if clock'event and clock = '1' then
	     counter1 <= counter1 + 1 ;
	     counter2 <= counter1 + seed; --  + counter2 + seed;
      end if;
   end process;

   counter1_out <= counter1;
   counter2_out <= counter2;



   jtag_2: process(tck)
   begin
      if tck'event and tck = '1' then

          -- Sample Instruction Handler */
         if read_dr = '1' and uir = '1' then
            --shadow_reg <= counter1;
            sample_instr_counter <= sample_instr_counter + 1;
         end if;
         
         if feed_dr = '1' then
	        seed <= seed + 1;  
         elsif uppload_ir = '1' or adr_dr = '1' or read_dr = '1' then
           seed <= seed;                  
         end if;
 
         if read_dr = '1' and cdr = '1' then
            tmp_reg <= q;
         elsif read_dr = '1' and sdr = '1' then
            tmp_reg <= tdi & tmp_reg(7 downto 1);
         end if;

         -- Feed Instruction Handler */ OK
         if adr_dr = '1' and sdr = '1' then
            addr_reg <= tdi & addr_reg(7 downto 1);               
         end if;

         -- Bypass register */
         bypass_reg <= tdi;
      end if; 

   end process;


   process( read_dr, adr_dr, tmp_reg(0), bypass_reg, addr_reg(0))
   begin
      -- Node TDO Output */
      if read_dr = '1' then
         tdo <= tmp_reg(0);
      elsif adr_dr = '1' then
         tdo <= addr_reg(0);		-- Used to maintain the continuity of the scan chain.
      else
         tdo <= bypass_reg;
      end if;
   end process;


   -- instantiation of the vji mega functionc */
   jtag_inst2: sld_virtual_jtag
   generic map (
      sld_auto_instance_index  => "NONE",
      sld_instance_index       => 2,
      sld_ir_width             => 2,
      sld_sim_action           => "",
      sld_sim_n_scan           => 0,
      sld_sim_total_length     => 0,
      lpm_type                 => "sld_virtual_jtag"
   )
   port map(
      ir_out  => sample_instr_counter, -- input to megafunction
      ir_in   => ir_in,                -- output from mega function
      tck     => tck,                  -- output from mega function
      tdi     => tdi,                  -- output from mgafunction
      tdo     => tdo,                  --input to mega function

      virtual_state_cdr  => cdr,       -- output from mega function
      virtual_state_e1dr =>e1dr,       --	"
      virtual_state_e2dr =>e2dr,       -- 	"
      virtual_state_pdr  => pdr,       -- 	"
      virtual_state_sdr  => sdr,       --	"
      virtual_state_udr  => udr,       -- 	"

      virtual_state_uir  => uir,       -- 	"
      virtual_state_cir  => cir	       --	"
   );


   RAM_8x256 : altsyncram
   GENERIC MAP (
      address_aclr_a   => "NONE",
      address_aclr_b   => "NONE",
      address_reg_b    => "CLOCK1",
      indata_aclr_a    => "NONE",
      intended_device_family => "Cyclone",
      lpm_type         => "altsyncram",
      numwords_a       => 256,
      numwords_b       => 256,
      operation_mode   => "DUAL_PORT",
      outdata_aclr_b   => "NONE",
      outdata_reg_b    => "CLOCK1",
      power_up_uninitialized => "FALSE",
      rdcontrol_aclr_b => "NONE",
      rdcontrol_reg_b  => "CLOCK1",
      widthad_a        => 8,
      widthad_b        => 8,
      width_a          => 8,
      width_b          => 8,
      width_byteena_a  => 1,
      wrcontrol_aclr_a => "NONE"
   )
   PORT MAP (
      wren_a     => '1',
      clock0     => clock,
      clock1     => tck, 
      address_a  => counter1, 
      address_b  => addr_reg, 
      rden_b     => read_dr,
      data_a     => counter2,
      q_b        => q
   );   


end Verhalten;